local responsePrefix = "<Waitlist> "
local responseFilter = "^"..responsePrefix
local commandFilter = "^wl"
local outputPrefix = "|cffffff7fWaitlist:|r "
local outputFrame = 1
local verbose = true
local squelch = true
local altPrefix = ""
local altSuffix = ""

if type(WaitlistDB) ~= "string" then
	WaitlistDB = {}
end

local function getmain(alt)
	local note
	local total = GetNumGuildMembers(true)
	for i=1,total do 
		local name = GetGuildRosterInfo(i)
		if name == alt then
			note = string.lower(select(7, GetGuildRosterInfo(i)))
		end
	end
	for i=1,total do
		local name = string.lower(GetGuildRosterInfo(i))
		if note:find(name) then
			return name
		end
	end
	return nil
end

local function addname(name)
	if UnitInRaid(name) then
		local message = name.." is already in the raid"
		return message
	else
		for k,v in ipairs(WaitlistDB) do
			if v == name then
					local message = name.." is already on the waitlist"
					return message
			end
		end
	end
	table.insert(WaitlistDB, name)
	local message = name.." has been added to the waitlist"
	return message
end

local function clobber(name)
	for k,v in ipairs(WaitlistDB) do
		if v == name then
			table.remove(WaitlistDB, k)
			local message = name.." has been removed from the waitlist"
			return message
		end
	end
	return nil
end

local function listnames()
	if table.getn(WaitlistDB) == 0 then
		local message = "No one is on the wait list" 
		return message
	else
		local message = "Current Waitlist: "
		for k,v in ipairs(WaitlistDB) do
			message = message..v..", "
		end
		message = string.sub(message, 1, -3)
		return message
	end
	return nil
end

local function fixcase(small)
	if type(small) == "string" then
		return string.gsub(string.lower(small), "%a", string.upper, 1) 
	end
end

local function outgoingFilter()
	if arg1:find(responseFilter) then return squelch end
end

local function incomingFilter()
	local input = string.lower(arg1)
	if input:find(commandFilter) then 
		local response = nil
		if input:find("add") then
			response = addname(arg2)
			if verbose then _G["ChatFrame"..outputFrame]:AddMessage(outputPrefix..response) end
		elseif input:find("main") then
			local main = fixcase(getmain(arg2))
			if main then
				response = addname(main)
			else
				response = "Unable to find main's name on the guild roster."
			end

		elseif input:find("del") or input:find("remove") then
			response = clobber(arg2)
		elseif input:find("who") or input:find("list") or input:find("report") then
			response = listnames()
		else
			response = "Usage: whisper me 'wl add', 'wl main', 'wl del', 'wl remove', 'wl who', 'wl list', or 'wl report'"
		end
		if response then SendChatMessage(responsePrefix..response, "WHISPER", nil, arg2) end
		return squelch
	end
end

ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", outgoingFilter)
ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", incomingFilter)

local frame = CreateFrame("Frame")
frame:SetScript("OnEvent", function(self, event, msg)
	if event == "RAID_ROSTER_UPDATE" then
		for i = 1, GetNumRaidMembers() do
			clobber(UnitName("raid"..i))
		end
	end
end)

frame:RegisterEvent("RAID_ROSTER_UPDATE")

SLASH_WAITLIST1 = "/waitlist";
SLASH_WAITLIST2 = "/wl";
SlashCmdList["WAITLIST"] = 
   function(msg)
   	local output
	local input = string.lower(msg)
	if input:find("^add ") then
		output = addname(fixcase(string.trim(string.sub(input, string.len("add ")))))
	elseif input:find("^del ") then
		output = clobber(fixcase(string.trim(string.sub(input, string.len("del ")))))
	elseif input:find("^remove ") then
		output = clobber(fixcase(string.trim(string.sub(input, string.len("remove ")))))
	elseif input:find("^who") or input:find("^list")  or input:find("^report") or input:find("^announce") then
		output = listnames()
		if input:find("officer") and GetNumGuildMembers ~= 0 then
			SendChatMessage(responsePrefix..output, "OFFICER")
			output = nil
		elseif (input:find("guild") and GetNumGuildMembers ~= 0) or input:find("^announce") then
			SendChatMessage(responsePrefix..output, "GUILD")
			output = nil
			if input:find("^announce") then
				SendChatMessage(responsePrefix.."Whisper me 'wl add' to get on the waitlist, or 'wl help' for more commands", "GUILD")
			end
		elseif input:find("raid") and UnitInRaid("player") then
			SendChatMessage(responsePrefix..output, "RAID")
			output = nil
		elseif input:find("party") and GetNumPartyMembers ~= 0 then
			SendChatMessage(responsePrefix..output, "PARTY")
			output = nil
		elseif input:find("say") then
			SendChatMessage(responsePrefix..output, "SAY")
			output = nil
		end
	elseif input:find("^reset") or input:find("^clear") then
		WaitlistDB = {}
		output = "Waitlist has been reset"
	else
		output = "Usage: /wl | /waitlist add | del/remove | who/list/report | announce | reset/clear"
	end
	if output then
		_G["ChatFrame"..outputFrame]:AddMessage(outputPrefix..output)
	end
   end
